@@ -36,10 +36,10 @@ enum f_type {
   T_ENUM_RTS = 0x30,
   T_ENUM_BGP_ORIGIN = 0x31,
   T_ENUM_SCOPE = 0x32,
-  T_ENUM_RTC = 0x33,
-  T_ENUM_RTD = 0x34,
-  T_ENUM_ROA = 0x35,
-  T_ENUM_NETTYPE = 0x36,
+  T_ENUM_RTD = 0x33,
+  T_ENUM_ROA = 0x34,
+  T_ENUM_ASPA = 0x35,
+  T_ENUM_NET_TYPE = 0x36,
   T_ENUM_RA_PREFERENCE = 0x37,
   T_ENUM_AF = 0x38,
   T_ENUM_MPLS_POLICY = 0x39,
