@@ -182,11 +182,11 @@
  *	m4_dnl	This structure is returned from the linearizer (105).
  *	m4_dnl	For writing directly to this structure, use FID_LINE_IN.
  *
- *	m4_dnl		f_dump_line_item_FI_EXAMPLE(const struct f_line_item *item, const int indent)
+ *	m4_dnl		f_dump_line_item_FI_EXAMPLE(struct dump_request *dreq, const struct f_line_item *item, const int indent)
  *	m4_dnl		{
  *	m4_dnl	(104)	  [[ put it here ]]
  *	m4_dnl		}
- *	m4_dnl	This code dumps the instruction on debug. Note that the argument
+ *	m4_dnl	This code dumps the instruction via RDUMP. Note that the argument
  *	m4_dnl	is the linearized instruction; if the instruction has arguments,
  *	m4_dnl	their code has already been linearized and their value is taken
  *	m4_dnl	from the value stack.
@@ -496,7 +496,7 @@
     RESULT(T_BOOL, i, (v1.type != T_VOID) && !val_is_undefined(v1));
   }
 
-  METHOD_R(T_NET, type, T_ENUM_NETTYPE, i, v1.val.net->type);
+  METHOD_R(T_NET, type, T_ENUM_NET_TYPE, i, v1.val.net->type);
   METHOD_R(T_IP, is_v4, T_BOOL, i, ipa_is_ip4(v1.val.ip));
 
   /* Add initialized variable */
@@ -1606,6 +1606,22 @@
 
   }
 
+  INST(FI_ASPA_CHECK_EXPLICIT, 2, 1) {	/* ASPA Check */
+    NEVER_CONSTANT;
+    ARG(1, T_PATH);
+    ARG(2, T_BOOL);
+    RTC(3);
+    struct rtable *table = rtc->table;
+
+    if (!table)
+      runtime("Missing ASPA table");
+
+    if (table->addr_type != NET_ASPA)
+      runtime("Table type must be ASPA");
+
+    RESULT(T_ENUM_ASPA, i, [[ aspa_check(table, v1.val.ad, v2.val.i) ]]);
+  }
+
   INST(FI_FROM_HEX, 1, 1) {	/* Convert hex text to bytestring */
     ARG(1, T_STRING);
 
