@@ -35,7 +35,7 @@ struct pool {
   const char *name;
 };
 
-static void pool_dump(resource *);
+static void pool_dump(struct dump_request *, resource *);
 static void pool_free(resource *);
 static resource *pool_lookup(resource *, unsigned long);
 static struct resmem pool_memsize(resource *P);
@@ -51,8 +51,6 @@ static struct resclass pool_class = {
 
 pool root_pool;
 
-static int indent;
-
 /**
  * rp_new - create a resource pool
  * @p: parent pool
@@ -100,16 +98,16 @@ pool_free(resource *P)
 }
 
 static void
-pool_dump(resource *P)
+pool_dump(struct dump_request *dreq, resource *P)
 {
   pool *p = (pool *) P;
   resource *r;
 
-  debug("%s\n", p->name);
-  indent += 3;
+  RDUMP("%s\n", p->name);
+  dreq->indent += 3;
   WALK_LIST(r, p->inside)
-    rdump(r);
-  indent -= 3;
+    rdump(dreq, r);
+  dreq->indent -= 3;
 }
 
 static struct resmem
@@ -199,20 +197,28 @@ rfree(void *res)
  * It works by calling a class-specific dump function.
  */
 void
-rdump(void *res)
+rdump(struct dump_request *dreq, void *res)
 {
   char x[16];
   resource *r = res;
 
-  bsprintf(x, "%%%ds%%p ", indent);
-  debug(x, "", r);
+  bsprintf(x, "%%%ds%%p ", dreq->indent);
+  RDUMP(x, "", r);
   if (r)
     {
-      debug("%s ", r->class->name);
-      r->class->dump(r);
+      RDUMP("%s ", r->class->name);
+      r->class->dump(dreq, r);
     }
   else
-    debug("NULL\n");
+    RDUMP("NULL\n");
+}
+
+void page_dump(struct dump_request *req);
+
+void resource_dump(struct dump_request *req)
+{
+  rdump(req, &root_pool);
+  page_dump(req);
 }
 
 struct resmem
@@ -251,6 +257,7 @@ ralloc(pool *p, struct resclass *c)
   return r;
 }
 
+#if 0
 /**
  * rlookup - look up a memory location
  * @a: memory address
@@ -273,6 +280,7 @@ rlookup(unsigned long a)
   else
     debug("Not found.\n");
 }
+#endif
 
 /**
  * resource_init - initialize the resource manager
@@ -316,11 +324,11 @@ static void mbl_free(resource *r UNUSED)
 {
 }
 
-static void mbl_debug(resource *r)
+static void mbl_debug(struct dump_request *dreq, resource *r)
 {
   struct mblock *m = (struct mblock *) r;
 
-  debug("(size=%d)\n", m->size);
+  RDUMP("(size=%d)\n", m->size);
 }
 
 static resource *
