@@ -122,6 +122,10 @@ struct bgp_config {
   /* Times below are in seconds */
   unsigned gr_time;			/* Graceful restart timeout */
   unsigned llgr_time;			/* Long-lived graceful restart stale time */
+  unsigned min_gr_time;			/* Minimum GR timeout */
+  unsigned max_gr_time;			/* Maximum GR timeout */
+  unsigned min_llgr_time;		/* Minimum LLGR stale time */
+  unsigned max_llgr_time;		/* Maximum LLGR stale time */
   unsigned connect_delay_time;		/* Minimum delay between connect attempts */
   unsigned connect_retry_time;		/* Timeout for connect attempts */
   unsigned hold_time;
@@ -146,7 +150,7 @@ struct bgp_config {
   int require_extended_messages;	/* Require remote support for extended messages [RFC 8654] */
   int require_hostname;			/* Require remote support for hostname [draft] */
   int require_gr;			/* Require remote support for graceful restart [RFC 4724] */
-  int require_llgr;			/* Require remote support for long-lived graceful restart [draft] */
+  int require_llgr;			/* Require remote support for long-lived graceful restart [RFC 9494] */
   struct bfd_options *bfd;		/* Use BFD for liveness detection */
 };
 
@@ -167,6 +171,8 @@ struct bgp_channel_config {
   u8 gr_able;				/* Allow full graceful restart for the channel */
   u8 llgr_able;				/* Allow full long-lived GR for the channel */
   uint llgr_time;			/* Long-lived graceful restart stale time */
+  uint min_llgr_time;			/* Minimum LLGR stale time */
+  uint max_llgr_time;			/* Maximum LLGR stale time */
   u8 ext_next_hop;			/* Allow both IPv4 and IPv6 next hops */
   u8 require_ext_next_hop;		/* Require remote support of IPv4 NLRI with IPv6 next hops [RFC 8950] */
   u8 add_path;				/* Use ADD-PATH extension [RFC 7911] */
@@ -241,7 +247,7 @@ struct bgp_af_caps {
   u8 ready;				/* Multiprotocol capability, RFC 4760 */
   u8 gr_able;				/* Graceful restart support, RFC 4724 */
   u8 gr_af_flags;			/* Graceful restart per-AF flags */
-  u8 llgr_able;				/* Long-lived GR, RFC draft */
+  u8 llgr_able;				/* Long-lived GR, RFC 9494 */
   u32 llgr_time;			/* Long-lived GR stale time */
   u8 llgr_flags;			/* Long-lived GR per-AF flags */
   u8 ext_next_hop;			/* Extended IPv6 next hop,   RFC 8950 */
@@ -261,7 +267,7 @@ struct bgp_caps {
   u8 gr_flags;				/* Graceful restart flags */
   u16 gr_time;				/* Graceful restart time in seconds */
 
-  u8 llgr_aware;			/* Long-lived GR capability, RFC draft */
+  u8 llgr_aware;			/* Long-lived GR capability, RFC 9494 */
   u8 any_ext_next_hop;			/* Bitwise OR of per-AF ext_next_hop */
   u8 any_add_path;			/* Bitwise OR of per-AF add_path */
 
@@ -453,6 +459,7 @@ struct bgp_write_state {
   int add_path;
   int mpls;
   int sham;
+  int ignore_non_bgp_attrs;
 
   eattr *mp_next_hop;
   const adata *mpls_labels;
@@ -650,7 +657,7 @@ int bgp_get_attr(const struct eattr *e, byte *buf, int buflen);
 void bgp_get_route_info(struct rte *, byte *buf);
 int bgp_total_aigp_metric_(rte *e, u64 *metric, const struct adata **ad);
 
-byte * bgp_bmp_encode_rte(struct bgp_channel *c, byte *buf, const net_addr *n, const struct rte *new, const struct rte_src *src);
+byte * bgp_bmp_encode_rte(struct bgp_channel *c, byte *buf, byte *end, const net_addr *n, const struct rte *new, const struct rte_src *src);
 
 #define BGP_AIGP_METRIC		1
 #define BGP_AIGP_MAX		U64(0xffffffffffffffff)
